#ifndef __CLIENT_H
#define __CLIENT_H

#include "tbuffer.h"
#include "network.h"

struct _Client {
	Client *prev, *next;

        IAuthListener *listener;
        time_t created;
	int handle;

	IPADDR ip;
        int port;

        TBuffer *inbuf, *outbuf;
	IO io;

	int auth_refcount, register_refcount, input_refcount;
	int module_data_size;
        void **module_data;
	unsigned int closed:1;
	unsigned int accepted:1; /* client is already passed to ircd */
	unsigned int disconnected:1;
	unsigned int registered:1;
	unsigned int rejected:1;
	unsigned int banned:1; /* k-lined */
	unsigned int nick_given:1; /* NICK command given (kludge for m_login) */
	unsigned int user_given:1; /* USER command given (kludge for m_login) */

	unsigned char *extra_input_data; /* points in inbuf->buffer+xx */
        int extra_input_size;

	IAuthClientInfo info;
};

extern Pool client_pool;

/* Create new client connection */
Client *client_create(int handle, IPADDR *ip, int port,
		      IAuthListener *listener);

/* Each module should call when beginning/ending the authentication. Once
   the refcount drops to zero, the authentication is finished and client
   is passed to ircd. */
void client_auth_ref(Client *client);
void client_auth_unref(Client *client);

/* Modules which want to be finished before client_registered is called
   (eg. ident, dns, login) should call these. client_auth_*() calls are
   not needed if you use these. */
void client_register_ref(Client *client);
void client_register_unref(Client *client);

/* Modules which want to read data from clients should call these. When the
   refcount drops to zero, the data isn't read anymore from client and any
   extra data not being parsed yet is sent to ircd. Note that you still must
   do auth/register referencing. */
void client_input_ref(Client *client);
void client_input_unref(Client *client);

/* Abort authentication, closes the handle. */
void client_auth_abort(Client *client, const char *msg);

/* Send a line to client */
void client_send_data(Client *client, const char *line);

void clients_init(void);
void clients_deinit(void);

#endif
