#ifndef __IAUTH_H
#define __IAUTH_H

#define IAUTH_CMD_MODULE_ADD		'A' /* <module>,[...] */
#define IAUTH_CMD_MODULE_REMOVE		'B' /* <module>,[...] */
#define IAUTH_CMD_MYINFO		'M' /* <server name> <version> <flags> <sid> :<info> */
#define IAUTH_CMD_DIE			'D' /* no parameters */
#define IAUTH_CMD_LISTEN_ADD		'L' /* <port> [<ip>] */
#define IAUTH_CMD_LISTEN_REMOVE		'R' /* <port> [<ip>] */
#define IAUTH_CMD_CONNECT_SERVER	'C' /* <host> <port> */

#define IAUTH_REPLY_REJECT		'R' /* <type> <user@host> <reason> */
#define IAUTH_REPLY_KILL		'K' /* <uid> <reason> */

#include "../irc/irc-limits.h"

/* for iauth -> ircd fd passing */
typedef struct {
	unsigned int id;
	unsigned int extra_input_size;

        /* modules may change the values below */
	char ip[IAUTH_IP_LENGTH+1];
	char nick[IAUTH_NICK_LENGTH+1];
	char user[IAUTH_USER_LENGTH+1];
	char host[IAUTH_HOST_LENGTH+1];
	char realname[IAUTH_REALNAME_LENGTH+1];

        char pass[IAUTH_PASS_LENGTH+1];
        char version[IAUTH_VERSION_LENGTH+1];
        char flags[IAUTH_FLAGS_LENGTH+1];
        char options[IAUTH_OPTIONS_LENGTH+1];
        char servername[IAUTH_SERVERNAME_LENGTH+1];
        char sid[IAUTH_SID_LENGTH+1];
        char serverinfo[IAUTH_SERVERINFO_LENGTH+1];

	int hopcount;
	int connect_port; /* if connect is TRUE */
	unsigned int restricted:1;
	unsigned int connect:1;
} IAuthClientInfo;

/* Called with client is registered (ident, DNS, PASS/NICK/USER/SERVER done) */
void iauth_client_registered(Client *client);

/* Called when client authentication is finished */
void iauth_client_authenticated(Client *client);

/* Called with client is rejected */
void iauth_client_rejected(Client *client, const char *msg);

#endif
