#ifndef __MODULES_H
#define __MODULES_H

struct _IAuthModule {
	char *name;

        /* Returns FALSE if failed */
        int (*init)(void);
	void (*deinit)(void);

        /* Connection accept()ed - must not start any stateful operation */
	void (*connection_accepted)(Client *client);
	/* All user authentication is finished (eg. ident, DNS, NICK, USER) */
	void (*client_registered)(Client *client);

	/* -- Operations below are stateful -- */

        /* Set *data to non-NULL to begin session. */
	void (*client_connected)(Client *client, void **data);
	/* Client was rejected - module should forget
	   everything that has to do with the client */
	void (*client_rejected)(Client *client, void *data);
	/* Client was accepted and passed to ircd - module may still continue
	   it's checks in background and tell the ircd later to disconnect
	   the client */
	void (*client_accepted)(Client *client, void *data);

	/* Timeout exceeded when waiting for authentication to finish */
        void (*client_timeout)(Client *client, void *data);
	/* Client sent a line. Returns TRUE if it was handled. */
	int (*client_read_line)(Client *client, const char *cmd,
				int argc, const char *args[], void *data);

	/* Data below initialized by module_register() */
	IAuthModule *next;
        int refnum;
	int refcount;
	unsigned int unregistered:1;
};

extern int modules_max_refnum;

/* Register module. IAuthModule must not be free'd. */
void module_register(IAuthModule *module);
/* Register a built-in module by name. */
void module_register_name(const char *name);
/* Unregister module. New clients won't use this module anymore, but it's
   used for the currently existing clients. */
void module_unregister(const char *name);

/* Call the hooks */
void module_connection_accepted(Client *client);
void module_client_connected(Client *client);
void module_client_rejected(Client *client);
void module_client_registered(Client *client);
void module_client_accepted(Client *client);
void module_client_timeout(Client *client);
int module_client_read_line(Client *client, const char *cmd,
			    int argc, const char *args[]);

void module_session_stop(IAuthModule *module, Client *client);

void modules_init(void);
void modules_deinit(void);

#endif
