/*
 m_ping.c : send ping with random data to client and require pong reply

    Copyright (C) 2002 Timo Sirainen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "common.h"

static void client_connected(Client *client, void **data);
static void client_rejected(Client *client, void *data);
static void client_timeout(Client *client, void *data);
static int client_read_line(Client *client, const char *cmd,
			    int argc, const char *args[], void *data);

IAuthModule m_ping = {
	"ping",

	NULL, /* init */
	NULL, /* deinit */

	NULL, /* connection_accepted */
	NULL, /* client_registered */

	client_connected,
	client_rejected,
        NULL, /* client_accepted */

        client_timeout,
	client_read_line
};


static void client_connected(Client *client, void **data)
{
	do {
		*data = INT_TO_POINTER(rand());
	} while (*data == NULL);

	client_auth_ref(client);
}

static void client_rejected(Client *client, void *data)
{
	client_auth_unref(client);
}

static void client_timeout(Client *client, void *data)
{
	/* still no correct PONG reply, abort */
	client_auth_abort(client, "timeout");
}

static int client_read_line(Client *client, const char *cmd,
			    int argc, const char *args[], void *data)
{
        const char *idstr;
        unsigned int id;

        id = (unsigned int) POINTER_TO_INT(data);

	if (strcmp(cmd, "NICK") == 0) {
                /* client connection, send the PING query */
		client_send_data(client, t_strdup_printf("PING :%x", id));
	} else if (strcmp(cmd, "SERVER") == 0) {
		/* server connection, no PING/PONG */
                module_session_stop(&m_ping, client);
		client_auth_unref(client);
	} else if (strcmp(cmd, "PONG") == 0) {
		idstr = argc > 1 ? args[1] : args[0];
		if (idstr != NULL &&
		    (unsigned int) strtol(idstr, NULL, 16) == id) {
			/* PONG ok */
			module_session_stop(&m_ping, client);
			client_auth_unref(client);
		}

                return TRUE;
	}

        return FALSE;
}
