#ifndef __IRC_LIMITS_H
#define __IRC_LIMITS_H

/* Hardcoded limits - these are set high enough so you most probably
   don't need to change them. */
#define LOGIN_MAX_FIELD_LENGTH		128 /* nick/user/realname */
#define COMMAND_MAX_ARGS		15
#define CLIENT_POOL_MIN_SIZE		2048 /* + the max. size of input buffer */
#define CLIENT_MAX_LINE_LENGTH		510
#define IAUTH_LOGIN_TIMEOUT		30 /* FIXME: should be runtime configurable.. */
#define IAUTH_CONNECT_TIMEOUT		60 /* FIXME: should be runtime configurable.. */

/* these are used with iauth communication, they're not used elsewhere: */
#define IAUTH_IP_LENGTH			46 /* INET6_ADDRSTRLEN */
#define IAUTH_NICK_LENGTH		30
#define IAUTH_USER_LENGTH		10
#define IAUTH_HOST_LENGTH		80
#define IAUTH_REALNAME_LENGTH		80
#define IAUTH_PASS_LENGTH		30
#define IAUTH_VERSION_LENGTH		30
#define IAUTH_FLAGS_LENGTH		20
#define IAUTH_OPTIONS_LENGTH		20
#define IAUTH_SERVERNAME_LENGTH		80
#define IAUTH_SID_LENGTH		20
#define IAUTH_SERVERINFO_LENGTH		80

/* server/nick ID lengths - server ID MUST begin with a digit */
#define UID_MAX_LENGTH			9
#define SERVER_MAX_ID			0x9ff
#define NICK_MAX_ID			0xffffff

/* Default limits - these can be changed from configuration file and
   rehashed on the fly, so you don't need to touch these directly */

/* Servers */
#define DEFAULT_SERVER_MAX_LINE_LENGTH	510 /* + 2 for \r\n */
#define DEFAULT_SERVER_MAX_MODE_ARGS	3

/* channels */
#define DEFAULT_CHANNEL_MAX_NAME	80
#define DEFAULT_CHANNEL_MAX_TOPIC	255
#define DEFAULT_CHANNEL_MAX_KEY		80 /* channel key (+k) */
#define DEFAULT_CHANNEL_MAX_KICKREASON 	80
#define DEFAULT_CHANNEL_MAX_PARTREASON 	80

/* clients */
#define DEFAULT_CLIENT_NICK_MAX_LENGTH	9
#define DEFAULT_CLIENT_USER_MAX_LENGTH  10 /* ident char + 9 chars */
#define DEFAULT_CLIENT_REALNAME_MAX_LENGTH 80
#define DEFAULT_CLIENT_MAX_AWAY_MSG	80
#define DEFAULT_CLIENT_MAX_INVITES	10
#define DEFAULT_CLIENT_INBUF_SIZE	1024
#define DEFAULT_CLIENT_MAX_OUTBUF_SIZE	4096 /* default for client class */
#define DEFAULT_CLIENT_MAX_CHANNELS	20
#define DEFAULT_CLIENT_PING_FREQUENCY	90
#define DEFAULT_CLIENT_MAX_PONG_WAIT	180

/* max. actions per command */
#define DEFAULT_CLIENT_MAX_MODE_ARGS	3
#define DEFAULT_CLIENT_MAX_WHOIS_NICKS	4
#define DEFAULT_CLIENT_MAX_KICK_TARGETS	4
#define DEFAULT_CLIENT_MAX_MSG_TARGETS	5
#define DEFAULT_CLIENT_MAX_NOTICE_TARGETS 5

#define DEFAULT_CLIENT_LOGIN_INBUF_SIZE	512
#define DEFAULT_CLIENT_LOGIN_OUTBUF_SIZE 512

#endif
