/*
 irc-strings.c : string functions for IRC based strings

    Copyright (C) 2002 Timo Sirainen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "irc-strings.h"

static const unsigned char rfc1459_upper[] = {
	0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09,
	0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f, 0x10, 0x11, 0x12, 0x13,
	0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d,
	0x1e, 0x1f,

	' ', '!', '"', '#', '$', '%', '&', '\'', '(', ')',
	'*', '+', ',', '-', '.', '/', '0', '1', '2', '3',
	'4', '5', '6', '7', '8', '9', ':', ';', '<', '=',
	'>', '?', '@', 'A', 'B', 'C', 'D', 'E', 'F', 'G',
	'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q',
	'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', '[',
	'\\', ']', '^', '_', '`', 'A', 'B', 'C', 'D', 'E',
	'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O',
	'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y',
	'Z', '[', '\\', ']', '^', /* uppercased {|}~ */ 0x7f,

	0x80, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87, 0x88, 0x89,
	0x8a, 0x8b, 0x8c, 0x8d, 0x8e, 0x8f, 0x90, 0x91, 0x92, 0x93,
	0x94, 0x95, 0x96, 0x97, 0x98, 0x99, 0x9a, 0x9b, 0x9c, 0x9d,
	0x9e, 0x9f, 0xa0, 0xa1, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7,
	0xa8, 0xa9, 0xaa, 0xab, 0xac, 0xad, 0xae, 0xaf, 0xb0, 0xb1,
	0xb2, 0xb3, 0xb4, 0xb5, 0xb6, 0xb7, 0xb8, 0xb9, 0xba, 0xbb,
	0xbc, 0xbd, 0xbe, 0xbf, 0xc0, 0xc1, 0xc2, 0xc3, 0xc4, 0xc5,
	0xc6, 0xc7, 0xc8, 0xc9, 0xca, 0xcb, 0xcc, 0xcd, 0xce, 0xcf,
	0xd0, 0xd1, 0xd2, 0xd3, 0xd4, 0xd5, 0xd6, 0xd7, 0xd8, 0xd9,
	0xda, 0xdb, 0xdc, 0xdd, 0xde, 0xdf, 0xe0, 0xe1, 0xe2, 0xe3,
	0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9, 0xea, 0xeb, 0xec, 0xed,
	0xee, 0xef, 0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7,
	0xf8, 0xf9, 0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xff
};

const unsigned char ascii_upper[] = {
	0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09,
	0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f, 0x10, 0x11, 0x12, 0x13,
	0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d,
	0x1e, 0x1f,

	' ', '!', '"', '#', '$', '%', '&', '\'', '(', ')',
	'*', '+', ',', '-', '.', '/', '0', '1', '2', '3',
	'4', '5', '6', '7', '8', '9', ':', ';', '<', '=',
	'>', '?', '@', 'A', 'B', 'C', 'D', 'E', 'F', 'G',
	'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q',
	'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', '[',
	'\\', ']', '^', '_', '`', 'A', 'B', 'C', 'D', 'E',
	'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O',
	'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y',
	'Z', '{', '|', '}', '~', 0x7f,

	0x80, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87, 0x88, 0x89,
	0x8a, 0x8b, 0x8c, 0x8d, 0x8e, 0x8f, 0x90, 0x91, 0x92, 0x93,
	0x94, 0x95, 0x96, 0x97, 0x98, 0x99, 0x9a, 0x9b, 0x9c, 0x9d,
	0x9e, 0x9f, 0xa0, 0xa1, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7,
	0xa8, 0xa9, 0xaa, 0xab, 0xac, 0xad, 0xae, 0xaf, 0xb0, 0xb1,
	0xb2, 0xb3, 0xb4, 0xb5, 0xb6, 0xb7, 0xb8, 0xb9, 0xba, 0xbb,
	0xbc, 0xbd, 0xbe, 0xbf, 0xc0, 0xc1, 0xc2, 0xc3, 0xc4, 0xc5,
	0xc6, 0xc7, 0xc8, 0xc9, 0xca, 0xcb, 0xcc, 0xcd, 0xce, 0xcf,
	0xd0, 0xd1, 0xd2, 0xd3, 0xd4, 0xd5, 0xd6, 0xd7, 0xd8, 0xd9,
	0xda, 0xdb, 0xdc, 0xdd, 0xde, 0xdf, 0xe0, 0xe1, 0xe2, 0xe3,
	0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9, 0xea, 0xeb, 0xec, 0xed,
	0xee, 0xef, 0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7,
	0xf8, 0xf9, 0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xff
};

/* yes, currently all of the tables are identical, but in future they
   may be different and it's better to keep them separated anyway to
   keep code more clear. */
const unsigned char *irc_id_uptable = rfc1459_upper;
const unsigned char *irc_nick_uptable = rfc1459_upper;
const unsigned char *irc_channel_uptable = rfc1459_upper;
const unsigned char *irc_mask_uptable = rfc1459_upper;

const unsigned char *irc_case_uptable = ascii_upper;


void irc_strings_set_rfc1459(int set)
{
	if (set) {
		irc_id_uptable = rfc1459_upper;
		irc_nick_uptable = rfc1459_upper;
		irc_channel_uptable = rfc1459_upper;
		irc_mask_uptable = rfc1459_upper;
	} else {
		irc_id_uptable = ascii_upper;
		irc_nick_uptable = ascii_upper;
		irc_channel_uptable = ascii_upper;
		irc_mask_uptable = ascii_upper;
	}
}

#define IRC_COMPARE(table, str1, str2) \
	int diff;                                               \
                                                                \
	while (*str1 != '\0' && *str2 != '\0') {                \
		diff = (table)[(int) (unsigned char) *str1] -   \
			(table)[(int) (unsigned char) *str2];   \
		if (diff != 0)                                  \
			return diff;                            \
		str1++;                                         \
		str2++;                                         \
	}                                                       \
                                                                \
       	return (table)[(int) (unsigned char) *str1] -           \
       		(table)[(int) (unsigned char) *str2];

#define IRC_HASH(table, p) \
	const unsigned char *s = p;                             \
	unsigned int g, h = 0;                                  \
                                                                \
	while (*s != '\0') {                                    \
		h = (h << 4) +                                  \
			(table)[(int) (unsigned char) *s];      \
		if ((g = h & 0xf0000000UL)) {                   \
			h = h ^ (g >> 24);                      \
			h = h ^ g;                              \
		}                                               \
		s++;                                            \
	}                                                       \
                                                                \
	return h /* % M */;                                     \


int irc_id_cmp(const char *id1, const char *id2)
{
        IRC_COMPARE(irc_id_uptable, id1, id2);
}

int irc_nick_cmp(const char *nick1, const char *nick2)
{
        IRC_COMPARE(irc_nick_uptable, nick1, nick2);
}

int irc_channel_cmp(const char *channel1, const char *channel2)
{
        IRC_COMPARE(irc_channel_uptable, channel1, channel2);
}

int irc_mask_cmp(const char *mask1, const char *mask2)
{
        IRC_COMPARE(irc_mask_uptable, mask1, mask2);
}

int irc_case_cmp(const char *str1, const char *str2)
{
        IRC_COMPARE(irc_case_uptable, str1, str2);
}

unsigned int irc_id_hash(const void *id)
{
        IRC_HASH(irc_id_uptable, id);
}

unsigned int irc_nick_hash(const void *nick)
{
        IRC_HASH(irc_nick_uptable, nick);
}

unsigned int irc_channel_hash(const void *channel)
{
	IRC_HASH(irc_channel_uptable, channel);
}

/* irc_char_attrs table based on the CharAttrs table from ircd-hybrid:
 *
 *   Copyright (C) 1990 Jarkko Oikarinen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Changes:
 * Thomas Helvey <tomh@inxpress.net> June 23, 1999
 *
 */
const unsigned int irc_char_attrs[] = {
/* 0-f */	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
/* 10-1f */	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
/* SP */     0,
/* ! */      0,
/* " */      0,
/* # */      CHANPREFIX_CHAR,
/* $ */      USER_CHAR,
/* % */      0,
/* & */      CHANPREFIX_CHAR,
/* ' */      0,
/* ( */      0,
/* ) */      0,
/* * */      SERVICE_CHAR,
/* + */      CHANPREFIX_CHAR,
/* , */      0,
/* - */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* . */      USER_CHAR|HOST_CHAR|SERVICE_CHAR,
/* / */      0,
/* 0 */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* 1 */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* 2 */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* 3 */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* 4 */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* 5 */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* 6 */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* 7 */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* 8 */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* 9 */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* : */      HOST_CHAR,
/* ; */      0,
/* < */      0,
/* = */      0,
/* > */      0,
/* ? */      0,
/* @ */      0,
/* A */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* B */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* C */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* D */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* E */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* F */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* G */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* H */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* I */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* J */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* K */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* L */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* M */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* N */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* O */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* P */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* Q */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* R */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* S */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* T */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* U */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* V */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* W */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* X */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* Y */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* Z */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* [ */      NICK_CHAR|USER_CHAR,
/* \ */      NICK_CHAR|USER_CHAR,
/* ] */      NICK_CHAR|USER_CHAR,
/* ^ */      NICK_CHAR|USER_CHAR,
/* _ */      NICK_CHAR|USER_CHAR,
/* ` */      NICK_CHAR|USER_CHAR,
/* a */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* b */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* c */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* d */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* e */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* f */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* g */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* h */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* i */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* j */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* k */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* l */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* m */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* n */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* o */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* p */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* q */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* r */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* s */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* t */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* u */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* v */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* w */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* x */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* y */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* z */      NICK_CHAR|USER_CHAR|HOST_CHAR,
/* { */      NICK_CHAR|USER_CHAR,
/* | */      NICK_CHAR|USER_CHAR,
/* } */      NICK_CHAR|USER_CHAR,
/* ~ */      USER_CHAR,
/* del */	0,
/* 80-8f */	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
/* 90-9f */	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
/* a0-af */	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
/* b0-bf */	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
/* c0-cf */	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
/* d0-df */	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
/* e0-ef */	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
/* f0-ff */	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};
