#ifndef __IRC_STRINGS_H
#define __IRC_STRINGS_H

/* Do we want RFC-1459 style string comparing?
   If not, use regular a-z = A-Z */
void irc_strings_set_rfc1459(int set);

/* Like strcasecmp(), but use the RFC-1459 case comparing optionally */
int irc_id_cmp(const char *id1, const char *id2);
int irc_nick_cmp(const char *nick1, const char *nick2);
int irc_channel_cmp(const char *channel1, const char *channel2);
int irc_mask_cmp(const char *mask1, const char *mask2);

/* like strcasecmp(), but make sure it's only a-z <-> A-Z case comparing */
int irc_case_cmp(const char *str1, const char *str2);

/* hash functions */
unsigned int irc_id_hash(const void *id);
unsigned int irc_nick_hash(const void *nick);
unsigned int irc_channel_hash(const void *channel);

#define NICK_CHAR		0x01
#define USER_CHAR		0x02
#define HOST_CHAR		0x04
#define SERVICE_CHAR		(0x08 | NICK_CHAR)
#define CHANPREFIX_CHAR		0x10

#define is_nick_char(c) \
	((irc_char_attrs[(int) (unsigned char) c] & NICK_CHAR) != 0)
#define is_service_char(c) \
	((irc_char_attrs[(int) (unsigned char) c] & SERVICE_CHAR) != 0)
#define is_user_char(c) \
	((irc_char_attrs[(int) (unsigned char) c] & NICK_USER) != 0)
#define is_host_char(c) \
	((irc_char_attrs[(int) (unsigned char) c] & HOST_CHAR) != 0)
#define is_chanprefix_char(c) \
	((irc_char_attrs[(int) (unsigned char) c] & CHANPREFIX_CHAR) != 0)

extern const unsigned char *irc_id_uptable;
extern const unsigned char *irc_nick_uptable;
extern const unsigned char *irc_channel_uptable;
extern const unsigned char *irc_mask_uptable;

extern const unsigned char *irc_case_uptable;
extern const unsigned int irc_char_attrs[];

#endif
