/*
 match.c : wildcard matching, assumes we're comparing IRC masks.

    Copyright (C) 2002 Timo Sirainen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "lib.h"
#include "irc-strings.h"

#define match_toupper(c) irc_mask_uptable[(int) (unsigned char) (c)]

static int mask_match_int(const char *mask, const char *data, int wild_start)
{
	const char *mask_start, *next_data;
	char datachr;
        int ok;

        mask_start = mask;
	do {
		if (wild_start) {
			/* first search for a position where the
			   first letters match */
			while (*mask != '\0' && *data != '\0') {
				if (match_toupper(*data) ==
				    match_toupper(*mask)) {
					data++; mask++;
					break;
				}
				data++;
			}
		}

		next_data = NULL; ok = TRUE;
		for (; *mask != '\0' && *data != '\0'; mask++) {
			if (*mask == '*') {
				/* *???.fi - match only hosts with 3+ chars */
				mask++;
				for (;; mask++) {
					if (*mask == '?') {
						if (*data == '\0') {
							ok = FALSE;
                                                        break;
						}
						data++;
					} else if (*mask != '*')
						break;
				}

				if (!ok) {
                                        /* non-match */
                                        break;
				}

				if (*mask == '\0')
					return TRUE;

				/* FIXME: hmm. if this works properly, we
				   could get rid of this recursion by simply
				   changing mask_start/next_data/wild_start */
				return mask_match_int(mask, data, TRUE);
			}

                        datachr = match_toupper(*data);
			if (wild_start && next_data == NULL &&
			    match_toupper(*mask_start) == datachr)
				next_data = data;

			if (*mask != '?' && match_toupper(*mask) != datachr) {
				/* non-match */
				ok = FALSE;
				break;
			}

			data++;
		}

		if (ok) {
			while (*mask == '*') mask++;
			if (*data == '\0' && *mask == '\0')
				return TRUE;
		}

		if (next_data == NULL && *data != '\0')
			data++;
		else
                        data = next_data;

		mask = mask_start;
	} while (data != NULL);

        return FALSE;
}

int mask_match(const char *mask, const char *data)
{
        return mask_match_int(mask, data, FALSE);
}

int is_mask(const char *mask)
{
	while (*mask != '\0') {
		if (*mask == '*' || *mask == '?')
                        return TRUE;
                mask++;
	}

        return FALSE;
}
