#ifndef __IRC_NETWORK_H
#define __IRC_NETWORK_H

struct _IrcNetwork {
	char *name;

        unsigned int logging_count;
	unsigned int users_count;
	unsigned int operators_count;
        unsigned int channels_count;

	unsigned int services_count;
	unsigned int servers_count;

	HashTable *servers; /* server name -> Server mappings */
	HashTable *server_ids; /* SID -> Server mappings */
	HashTable *nicks; /* nick name -> Nick mappings */
        HashTable *nick_ids; /* nick ID -> Nick mappings */
	HashTable *channels;

	Server *server; /* our server record */
	Server *primary_link; /* our primary link to outworld,
		if this breaks try to connect to alternative links */

	/* list of capabilities that none/all of our links have.
	   can easily be used to check if we should bother doing something
	   that doesn't need to be done for any server. */
	unsigned int no_caps;
	unsigned int all_caps;

	unsigned int min_line_length; /* smallest max. line length in
		our server links */
};

IrcNetwork *irc_network_create(const char *name);
void irc_network_destroy(IrcNetwork *net);

void irc_network_disconnect(IrcNetwork *net, const char *msg);

/* Find nick by name/id */
Nick *ircnet_find_nick(IrcNetwork *net, const char *id);
/* Find nick by name */
Nick *ircnet_find_nick_name(IrcNetwork *net, const char *nick);
/* Find nick by ID */
Nick *ircnet_find_nick_id(IrcNetwork *net, const char *id);

/* Returns the server record for given server name/ID. */
Server *ircnet_find_server(IrcNetwork *net, const char *server);
/* Returns the server record for given server name. */
Server *ircnet_find_server_name(IrcNetwork *net, const char *name);
/* Returns the server record for given server ID. */
Server *ircnet_find_server_id(IrcNetwork *net, const char *id);
/* Returns the server record for given server wildcard. */
Server *ircnet_find_server_wildcard(IrcNetwork *net, const char *mask);

/* Returns channel record for given channel name. */
Channel *ircnet_find_channel(IrcNetwork *net, const char *name);

/* Update no_caps and all_caps by checking the capabilities
   in currently linked servers. */
void ircnet_update_caps(IrcNetwork *net);

#define ircnet_links(net) ((net)->server->downlinks)

void irc_networks_init(void);
void irc_networks_deinit(void);

#endif
