#include "common.h"
#include "ioloop.h"
#include "lib-signals.h"
#include "settings.h"

#include "irc-network.h"
#include "server.h"
#include "server-connection.h"
#include "client.h"
#include "channel.h"

#include "client-class.h"
#include "client-login.h"

#include "server-commands.h"
#include "client-commands.h"

#include "iauth-connection.h"

static IOLoop ioloop;
static IrcNetwork *ircnet;

static void sig_quit(int signo)
{
        io_loop_stop(ioloop);
}

static void ircd_init(void)
{
	lib_init_signals(sig_quit);

        settings_init();

	irc_networks_init();
	servers_init();
        channels_init();
	clients_init();

        client_classes_init();
        client_logins_init();

        server_commands_init();
	client_commands_init();

        iauth_init();

        /* create our own server record */
	ircnet = settings_server_init();
	if (ircnet == NULL)
                exit(1);
}

static void ircd_deinit(void)
{
        if (lib_signal_kill != 0)
		i_warning("Killed with signal %d\n", lib_signal_kill);

        iauth_deinit();

	irc_network_disconnect(ircnet, "Server quit");
	irc_network_destroy(ircnet);

        client_commands_deinit();
	server_commands_deinit();

	client_logins_deinit();
        client_classes_deinit();

	clients_deinit();
        channels_deinit();
	servers_deinit();
        irc_networks_deinit();

        settings_deinit();
}

int main(int argc, char *argv[])
{
	if (argc > 1)
		config_file = argv[1];

        lib_init();
	ioloop = io_loop_create();

	ircd_init();
        io_loop_run(ioloop);
	ircd_deinit();

	io_loop_destroy(ioloop);
	lib_deinit();

        return 0;
}
