#ifndef __NICK_H
#define __NICK_H

#include "network.h"
#include "send.h"

#define USER_MODES_LIST		"aoOirw"

#define UMODE_AWAY		0x01
#define UMODE_IRCOP     	(0x02 | UMODE_LOCAL_IRCOP)
#define UMODE_LOCAL_IRCOP     	0x04
#define UMODE_INVISIBLE		0x08
#define UMODE_RESTRICTED	0x10
#define UMODE_WALLOPS		0x20

struct _Nick {
        Server *server;
        int refcount;

	char *output_id; /* \rUID, or nick if server doesn't support UIDs */
	char *nick;

	time_t created; /* time when the nick joined to IRC network,
		or if not known, the record creation time */
        time_t last_nick_change; /* or created-time if never changed. */

	char *user;
	char *host;
	char *real_name;
	IPADDR ip;
        int hopcount;

        char *away_message;

	unsigned int mode;
	PArray channels; /* List of channels the nick has joined.
		May be NULL. */
        unsigned int leaf_servers; /* Leaf servers we're connected to that
                need to be notified about changes in this nick. This is a
		bitmask so we can have only BITS_IN_UINT leafs. */

	unsigned int removed:1; /* TRUE after server_remove_nick() */
};

/* Create a new nick record, and add it to server. The memory is allocated
   from server's pool. Don't use for local clients. */
Nick *nick_create(Server *server, const char *id, const char *nick_name,
		  const char *user, const char *host, const char *real_name);
/* Reference nick record. */
void nick_ref(Nick *nick);
/* Unreference nick record. */
void nick_unref(Nick *nick);

/* Changes the nick name. */
void nick_change(Nick *nick, const char *new_nick, time_t timestamp);

/* Returns the nick's user mode as string. */
const char *nick_get_umode_str(Nick *nick);
/* Changes the nick's user mode. */
void nick_set_umode(Nick *nick, const char *mode);
/* Returns nick!user@host */
const char *nick_get_mask(Nick *nick);

/* Returns the nick ID. */
#define nick_get_id(nick) \
	((nick)->output_id[0] == '\r' ? (nick)->output_id+1 : (nick)->output_id)
/* Returns the nick's string ID that can be used in all
   strings to reference the nick (\rUID) */
#define nick_get_outputid(nick) ((nick)->output_id)

/* Fix nickmask by adding missing "!*@*" or "@*" */
const char *nick_mask_fix(const char *mask);

/* Returns TRUE if nickname is valid. */
int nick_is_valid(const char *nick);

void nick_send_data(Nick *nick, SendOptions *sopts, const char *data);

/* INTERNAL: for client_destroy() */
void nick_destroy(Nick *nick);

#endif
