#ifndef __NUMERICS_H
#define __NUMERICS_H

#include "numeric-values.h"
#include "numeric-strings.h"

#define client_send_numeric_noargs(client, num) \
	client_send(client, NULL, t_strdup_printf(s_##num, client->nick.nick))

#define send_numeric_to_noargs(sopts, target, num) \
	send_to(sopts, t_strdup_printf(s_##num, target))

#if defined (_ISOC99_SOURCE) && !defined(USE_NUMERICS_ARRAY)
/* C99 */
#  define client_send_numeric(client, num, ...) \
	client_send(client, NULL, t_strdup_printf(s_##num, \
					client->nick.nick, __VA_ARGS__))
#  define send_numeric_reply(origin, num, ...) \
	send_command_reply(origin, t_strdup_printf(s_##num, \
				ORIGIN_GET_OUTPUTID(origin), __VA_ARGS__))
#  define send_numeric_to(sopts, target, num, ...) \
	send_to(sopts, t_strdup_printf(s_##num, target, __VA_ARGS__))
#elif defined (__GNUC__) && !defined (__STRICT_ANSI__) && \
	!defined(USE_NUMERICS_ARRAY)
/* GCC */
#  define client_send_numeric(client, num, args...) \
	client_send(client, NULL, t_strdup_printf(s_##num, \
						client->nick.nick, ##args))
#  define send_numeric_reply(origin, num, args...) \
	send_command_reply(origin, t_strdup_printf(s_##num, \
					ORIGIN_GET_OUTPUTID(origin), ##args))
#  define send_numeric_to(sopts, target, num, args...) \
	send_to(sopts, t_strdup_printf(s_##num, target, ##args))
#else
/* fallback - the only problem with this is that the compiler can't warn
   about format errors. In fact, we might even want to use this if compiler
   doesn't compress multiple identical string constants into one location */
#ifndef USE_NUMERICS_ARRAY
#  define USE_NUMERICS_ARRAY
#endif
void client_send_numeric(Client *client, int num, ...);
void send_numeric_reply(CommandOrigin *origin, int num, ...);
void send_numeric_to(SendOptions *sopts, const char *target, int num, ...);
#endif

#endif
