#ifndef __SEND_H
#define __SEND_H

typedef struct _CommandOrigin CommandOrigin;
typedef struct _NetSendBuffer NetSendBuffer;

typedef struct {
	Nick *origin_nick;
	Server *origin_server;
	const char *origin; /* don't use unless really needed.. */

	Server *skip_server; /* Don't send message to this server. */
        Client *skip_client; /* Don't send message to this client. */

	Server *target_server; /* Send only to this server */
	Client *target_client; /* Send only to this client */
	Nick *target_nick; /* target_server or target_client must be set too */

        unsigned int caps_set; /* skip server if it doesn't have these caps */
        unsigned int caps_unset; /* skip server if it has these caps */

	unsigned int channel_no_clients:1;
	unsigned int channel_no_servers:1;
	unsigned int channel_send_talk:1; /* sending PRIVMSG / NOTICE */

        time_t timestamp;
} SendOptions;

#define SOPTS_GET_TARGET(sopts) \
	((sopts)->target_nick != NULL ? (sopts)->target_nick->nick : \
	 (sopts)->target_server != NULL ? (sopts)->target_server->name : "*")

#define SOPTS_GET_OUTPUT_ID(sopts) \
	((sopts)->target_nick != NULL ? \
		nick_get_outputid((sopts)->target_nick) : \
	(sopts)->target_server != NULL ? \
		server_get_outputid((sopts)->target_server) : "*")

/* Initializes a SendOptions structure with given parameter. */
SendOptions *send_options_clear(SendOptions *opts);
SendOptions *send_origin(SendOptions *opts, const char *str);
SendOptions *send_origin_nick(SendOptions *opts, Nick *nick);
SendOptions *send_origin_nick_skip(SendOptions *opts, Nick *nick);
SendOptions *send_origin_client_skip(SendOptions *opts, Client *client);
SendOptions *send_origin_server(SendOptions *opts, Server *server);
SendOptions *send_origin_nickserver(SendOptions *opts, Nick *nick,
				    Server *server);
SendOptions *send_skip_server(SendOptions *opts, Server *skip_server);
SendOptions *send_to_client(SendOptions *opts, Client *client);
SendOptions *send_to_server(SendOptions *opts, Server *server);
SendOptions *send_to_nick(SendOptions *opts, Nick *nick);
SendOptions *send_back(SendOptions *opts, CommandOrigin *origin);

/* Send data to all servers and clients that need to know about
   changes in the nick. */
void send_nick_update(Nick *nick, SendOptions *opts, const char *data);

/* Send command to target_client/target_server */
void send_to(SendOptions *sopts, const char *data);

/* Send reply to origin (nick/server). */
void send_command_reply(CommandOrigin *origin, const char *data);

/* Send buffer - when the line gets full it's sent to server, each line
   begins with the given prefix. Note that server_sendbuf_add() might always
   cut the line there, so if you want to call it multiple times without
   cutting, call server_sendbuf_check_flush() first.

   NOTE: this sendbuffer uses temp. memory pool, so you shouldn't
   save it permanently anywhere. */
NetSendBuffer *ircnet_sendbuf_create(IrcNetwork *net, SendOptions *sopts,
				     const char *prefix, int max_line_size);
void ircnet_sendbuf_flush(NetSendBuffer *buf);
/* Make sure the buffer contains enough space, flush the buffer if not.
   Returns TRUE if the buffer was flushed or was originally empty. */
int ircnet_sendbuf_check(NetSendBuffer *buf, int need_size);
void ircnet_sendbuf_add(NetSendBuffer *buf, const char *data, int size);

#endif
