#ifndef __SERVER_COMMANDS_H
#define __SERVER_COMMANDS_H

#include "commands.h"

#define SERVER_COMMAND_EXT(cmdname) \
	void cmdname(CommandOrigin *origin, const char *cmd, \
		     int argc, const char *args[])

#define SERVER_COMMAND(cmdname) \
        static SERVER_COMMAND_EXT(cmdname)

#define SERVER_CURRENT_LINE(known_args, use_last) \
	get_current_line(origin->sconn->current_cmd_line, \
			 known_args, use_last, cmd, argc, args)

#define ORIGIN_GET_OUTPUTID(origin) \
	((origin)->nick != NULL ? nick_get_outputid((origin)->nick) : \
	server_get_outputid((origin)->server))

#define ORIGIN_GET_NAME(origin) \
	((origin)->nick != NULL ? (origin)->nick->nick : \
	(origin)->server->name)

struct _CommandOrigin {
	Nick *nick; /* The nick who sent this command (NULL if server) */
	Server *server; /* The server who sent this command (never NULL) */
	ServerConn *sconn; /* The server who forwarded this command to us. */
};

typedef void (*ServerCommandFunc) (CommandOrigin *origin, const char *cmd,
				   int argc, const char *args[]);

void server_command_register(const char *name, int min_params,
			     ServerCommandFunc func);
void server_command_unregister(const char *name, ServerCommandFunc func);

void server_command_parse(CommandOrigin *origin, const char *name,
			  int argc, const char *args[]);

/* Returns TRUE if the server is our local server. */
int command_forward_server(CommandOrigin *origin, const char *cmd,
			   const char *args[], int serverarg);

void server_commands_init(void);
void server_commands_deinit(void);

#endif
