#ifndef __SERVER_CONNECTION_H
#define __SERVER_CONNECTION_H

typedef struct _NickList {
        struct _NickList *prev, *next;
        time_t last_access;
	char *nick;
} NickAccessList;

struct _ServerConn {
        IrcNetwork *network;
	Server *server;

	int handle;
        IO io;
	int refcount;

	TBuffer *inbuf, *outbuf;
	HashTable *tokens; /* int their_token -> Server */
        int leaf_id; /* 0..BITS_IN_UINT, or -1 if we're not a leaf */

	/* Used only if connection is a leaf server. Contains a list of nicks
	   the server constantly requests information about. When user with
	   this nick joins to IRC, we notify the leaf of it. This allows
	   leaf to answer to ISON (used constantly for notify lists), etc.
	   commands immediately without asking the hub every time. When
	   nick is removed from this list because of n minutes of inactivity,
	   the leaf must be notified. */
	HashTable *unknown_nicks; /* points to NickAccessList */
	NickAccessList *first_unknown_nick; /* oldest item in the list */
        NickAccessList *last_unknown_nick; /* newest item in the list */

        time_t last_data; /* last time we received data from server */

	unsigned int compressed:1; /* the connection is gzip-compressed */
	unsigned int disconnected:1; /* the server just disconnected */
	unsigned int caps; /* server's capabilities (CAP_xxx) */
        unsigned int max_mode_args; /* max. MODE arguments */
	unsigned int max_line_length; /* max. length for line
		(not including \r\n\0) */

	const char *current_cmd_id; /* command reply ID for currently
		processed command */
	const char *current_cmd_line; /* command line without prefix */
	time_t current_cmd_timestamp; /* timestamp when currently processed
		command was sent from the originating server
		(with time_diff applied) */

	Pool pool; /* same as server->pool */
};

/* Called when iauth has sent us a new server connection.
   Creates a new server record for the connection. */
ServerConn *server_connect(Server *own_server, int handle, const char *id,
			   const char *name, const char *info);
/* Reference the server connection. */
void server_conn_ref(ServerConn *sconn);
/* Unreference the server connection. If the recount drops to zero,
   destroy the connetcion. Doesn't do any other deinitialization. */
void server_conn_unref(ServerConn *sconn);

/* Find the server connection for the specified server */
ServerConn *server_conn_find(Server *server);
/* Find the server connection for the link to the specified server */
ServerConn *server_conn_link_find(Server *server);

/* Set new capabilities to connection. You can't (and you shouldn't need
   to) remove the capabilities later. */
void server_conn_set_caps(ServerConn *sconn, unsigned int caps);

/* Set the token ID for the specified server used by the server at the
   other side of the connection. */
void server_conn_set_token(ServerConn *sconn, Server *dest_server,
			   unsigned int token);
/* Find the server for the given token. */
Server *server_conn_find_token(ServerConn *sconn, unsigned int token);

/* Add the nick to unknown_nicks list. If it already is there, just move
   it to last in the list. */
NickAccessList *server_leaf_access_unknown_nick(Server *server,
						const char *nick);
/* Remove the specified nick from list of unknown nicks. */
void server_leaf_remove_unknown_nick(Server *server, const char *nick);
/* Returns TRUE if the leaf server wants to know about the
   newly introduced nick (ie. is in unknown_nicks list). */
int server_leaf_find_unknown_nick(Server *server, const char *nick);

/* Disconnect the server with specified reason. Send the needed
   QUIT and SQUIT messages to clients and servers. */
void server_disconnect(ServerConn *sconn, const char *msg);
/* Server disconnected because of read/write error. */
void server_disconnect_errno(ServerConn *sconn);

/* INTERNAL: Parse/execute the input in server connection's receive buffer. */
void server_parse_input(ServerConn *sconn);

#endif
