#include "common.h"
#include "../lib-config/iconfig.h"

#include "iauth-connection.h"

static Config *config;

char *config_file = DEFAULT_CONFIG_FILE_PATH;

/* servers */
int server_max_line_length = DEFAULT_SERVER_MAX_LINE_LENGTH;
int server_max_mode_args = DEFAULT_SERVER_MAX_MODE_ARGS;

/* channels */
int channel_max_name = DEFAULT_CHANNEL_MAX_NAME;
int channel_max_topic = DEFAULT_CHANNEL_MAX_TOPIC;
int channel_max_key = DEFAULT_CHANNEL_MAX_KEY;
int channel_max_kickreason = DEFAULT_CHANNEL_MAX_KICKREASON;
int channel_max_partreason = DEFAULT_CHANNEL_MAX_PARTREASON;

/* clients */
int client_nick_max_length = DEFAULT_CLIENT_NICK_MAX_LENGTH;
int client_user_max_length = DEFAULT_CLIENT_USER_MAX_LENGTH;
int client_realname_max_length = DEFAULT_CLIENT_REALNAME_MAX_LENGTH;
int client_max_away_msg = DEFAULT_CLIENT_MAX_AWAY_MSG;
int client_max_invites = DEFAULT_CLIENT_MAX_INVITES;
int client_inbuf_size = DEFAULT_CLIENT_INBUF_SIZE;
int client_max_channels = DEFAULT_CLIENT_MAX_CHANNELS;
int client_ping_frequency = DEFAULT_CLIENT_PING_FREQUENCY;
int client_max_pong_wait = DEFAULT_CLIENT_MAX_PONG_WAIT;

int client_max_mode_args = DEFAULT_CLIENT_MAX_MODE_ARGS;
int client_max_whois_nicks = DEFAULT_CLIENT_MAX_WHOIS_NICKS;
int client_max_kick_targets = DEFAULT_CLIENT_MAX_KICK_TARGETS;
int client_max_msg_targets = DEFAULT_CLIENT_MAX_MSG_TARGETS;
int client_max_notice_targets = DEFAULT_CLIENT_MAX_NOTICE_TARGETS;

int client_login_inbuf_size = DEFAULT_CLIENT_LOGIN_INBUF_SIZE;
int client_login_outbuf_size = DEFAULT_CLIENT_LOGIN_OUTBUF_SIZE;

int client_cmd_wait = DEFAULT_CLIENT_CMD_WAIT;
int client_cmd_threshold = DEFAULT_CLIENT_CMD_THRESHOLD;

char *client_default_away_reason;
char *client_default_mode;

int channel_ts_max_diff = DEFAULT_CHANNEL_TS_MAX_DIFF;

static int *iauth_get_ports(ConfigBlock *iauth_block)
{
	ConfigList *list;
	int len, i, *arr;

	list = config_find_value(iauth_block, "ports");
	len = config_list_length(list);

	arr = t_malloc(len+1);
	for (i = 0; i < len; i++) {
		arr[i] = atoi(list->data);
		list = list->next;
	}
	arr[len] = 0;

        return arr;
}

static const char *iauth_get_modules(ConfigBlock *iauth_block)
{
	ConfigList *list, *tmp;
        char *modules;
	int len, slen;

	list = config_find_value(iauth_block, "modules");
	if (list == NULL)
                return NULL;

        /* calculate string length */
        len = 0;
	for (tmp = list; tmp != NULL; tmp = tmp->next) {
                len += strlen(tmp->data)+1;
	}

	/* create the string */
        modules = t_malloc(len);
        len = 0;
	for (tmp = list; tmp != NULL; tmp = tmp->next) {
		slen = strlen(tmp->data);

		memcpy(modules+len, tmp->data, slen);
                modules[len+slen] = ',';
		len += slen+1;
	}

	modules[len-1] = '\0';
        return modules;
}

IrcNetwork *settings_server_init(void)
{
        IrcNetwork *ircnet;
        ConfigBlock *block, **blocks;
	const char *name, *sid, *info;
        int i;

        block = config_find_block(config, NULL, "server");
	if (block == NULL) {
                fprintf(stderr, "Missing server block\n");
		return NULL;
	}

        /* read server data */
	name = block->name == NULL ? NULL : block->name->data;
        sid = config_find_value_first(block, "sid");
        info = config_find_value_first(block, "info");

	if (name == NULL || sid == NULL || info == NULL) {
                if (name == NULL)
			fprintf(stderr, "missing server name\n");
                if (sid == NULL)
			fprintf(stderr, "missing server ID\n");
                if (info == NULL)
			fprintf(stderr, "missing server info\n");
                return NULL;
	}

	ircnet = irc_network_create("ircnet");
	ircnet->server = server_create(ircnet, NULL, sid, name, info, 0);
	ircnet->server->local = TRUE;
        ircnet_update_caps(ircnet);

	/* read iauths */
	blocks = config_find_blocks(config, block, "iauth");
        memset(t_malloc(1000), 0xff, 1000);

	for (i = 0; blocks[i] != NULL; i++) {
		/* put modules to char* list */
		iauth_create(ircnet->server, NULL,
			     iauth_get_ports(blocks[i]),
			     iauth_get_modules(blocks[i]));
	}

        return ircnet;
}

void settings_init(void)
{
        client_default_away_reason = i_strdup(DEFAULT_CLIENT_AWAY_REASON);
	client_default_mode = i_strdup(DEFAULT_CLIENT_MODE);

	config = config_init(config_file);
	switch (config_parse(config)) {
	case -1:
		fprintf(stderr, "Can't open config file %s: %s\n",
			config_file, strerror(errno));
	case 0:
		exit(1);
	}
}

void settings_deinit(void)
{
	config_deinit(config);
        i_free(client_default_away_reason);
        i_free(client_default_mode);
}
