#include "lib.h"
#include "internal.h"

void config_list_destroy(Config *config, ConfigList *list)
{
	ConfigList *next;

	while (list != NULL) {
		next = list->next;
                p_free(config->pool, list->data);
                p_free(config->pool, list);
                list = next;
	}
}

static void config_block_destroy(Config *config, ConfigBlock *block);

static void config_items_destroy(Config *config, ConfigItem *items)
{
	ConfigItem *next;

	while (items != NULL) {
		next = items->next;

		switch (items->type) {
		case CONFIG_ITEM_TYPE_COMMENT:
			p_free(config->pool, items->value.comment);
                        break;
		case CONFIG_ITEM_TYPE_BLOCK:
                        config_block_destroy(config, items->value.block);
                        break;
		case CONFIG_ITEM_TYPE_VALUE:
                        config_list_destroy(config, items->value.list);
                        break;
		}

		p_free(config->pool, items->key);
		p_free(config->pool, items);
                items = next;
	}
}

static void config_block_destroy(Config *config, ConfigBlock *block)
{
	config_list_destroy(config, block->name);
	config_items_destroy(config, block->items);

        p_free(config->pool, block);
}

int config_list_length(ConfigList *list)
{
	int len;

        len = 0;
	while (list != NULL) {
                len++;
                list = list->next;
	}

        return len;
}

Config *config_init(const char *fname)
{
	Config *config;

	config = i_new(Config, 1);
	config->pool = pool_create("Config", 10240, TRUE);
	config->fname = p_strdup(config->pool, fname);
        config->root = p_new(config->pool, ConfigBlock, 1);
        return config;
}

void config_deinit(Config *config)
{
        config_block_destroy(config, config->root);
        p_free_clean(config->pool, config->fname);
	pool_unref(config->pool);
	i_free(config);
}
