#ifndef __CONFIG_PARSER_H
#define __CONFIG_PARSER_H

typedef struct _ConfigBlock ConfigBlock;

typedef struct _ConfigList {
	struct _ConfigList *next;
        char *data;
} ConfigList;

#define CONFIG_ITEM_TYPE_COMMENT	0 /* key = NULL, data = char *comment */
#define CONFIG_ITEM_TYPE_BLOCK		1 /* key = class, data = ConfigBlock */
#define CONFIG_ITEM_TYPE_VALUE		2 /* key = key, value = ConfigList */

typedef struct _ConfigItem {
	struct _ConfigItem *next;

	int type;
	char *key;
	union {
		char *comment;
		ConfigBlock *block;
                ConfigList *list;
	} value;
} ConfigItem;

struct _ConfigBlock {
	ConfigBlock *parent;

	ConfigList *name;
        ConfigItem *items;
};

typedef struct {
	char *fname;

	Pool pool;
        ConfigBlock *root;
} Config;

Config *config_init(const char *fname);
void config_deinit(Config *config);

/* Returns 1 = ok, 0 = error, -1 = error opening config file */
int config_parse(Config *config);
/* Write config to file, if fname == NULL, use the config->fname.
   Returns 0 = ok, else = errno */
int config_write(Config *config, const char *fname);

/* Find config block by key. If block is NULL, root is used. */
ConfigBlock *config_find_block(Config *config, ConfigBlock *block,
			       const char *key);
ConfigBlock **config_find_blocks(Config *config, ConfigBlock *block,
				 const char *key);

/* Find value from block */
ConfigList *config_find_value(ConfigBlock *block, const char *key);
char *config_find_value_first(ConfigBlock *block, const char *key);

int config_list_length(ConfigList *list);

#endif
