#ifndef __CLIENT_H
#define __CLIENT_H

#include "tbuffer.h"

struct _Client {
	struct _Client *prev, *next; /* not sorted */

        IAuth *iauth; /* which iauth was used to accept this client */
        unsigned int iauth_uid;

	IrcNetwork *network;
	Server *server;
	int handle; /* socket handle */
        IO io;

	TBuffer *inbuf, *outbuf;

	const char *current_cmd_line;

	char *nick, *user, *realname, *version;
	unsigned int logged_in:1;
	unsigned int disconnected:1;
};

/* Create a new client connection to specified server. */
Client *client_create(IrcNetwork *network, int handle, IPADDR *ip,
		      const char *nick, const char *user,
		      const char *host, const char *realname);
int client_destroy(Client *client);

/* FIXME: requires GCC */
#define client_send_numeric(client, num, args...) \
	client_send(client, t_strdup_printf(s_##num, client->nick, ##args))

void client_send(Client *client, const char *data);
void client_login(Client *client, Server *server);
void client_set_logged_in(Client *client);

void client_set_nick(Client *client, const char *nick);
void client_set_version(Client *client, const char *version);

/* Disconnect the client with specified reason. Client is notified with an
   ERROR message if notify_client is TRUE. Other clients and servers are
   notified with a QUIT message if notify_others is TRUE. */
void client_disconnect(Client *client, const char *msg);
/* Client disconnected because of read/write error.
   Notifies others with a QUIT message. */
void client_disconnect_errno(Client *client);

/* Disconnect all clients in given server. */
void clients_disconnect(const char *msg);

void clients_init(void);
void clients_deinit(void);

#endif
