#include "common.h"
#include "ioloop.h"
#include "tree.h"
#include "lib-signals.h"

#include "client.h"
#include "server.h"
#include "client-commands.h"
#include "iauth-connection.h"
#include "proxy-config.h"

static IAuth *iauth;
static IrcNetwork *network;
static IOLoop ioloop;

static void sig_quit(int signo)
{
        io_loop_stop(ioloop);
}

static void proxy_init(void)
{
	IPADDR ip, my_ip;

	lib_init_signals(sig_quit);

	clients_init();
	servers_init();
	client_commands_init();

	iauth_init();

        net_host2ip(NETWORK_IP, &ip);
#ifdef NETWORK_LISTEN_IP
        net_host2ip(NETWORK_LISTEN_IP, &my_ip);
#else
	memset(&my_ip, 0, sizeof(my_ip));
#endif

	network = irc_network_create(NETWORK_NAME, NETWORK_ADDRESS,
				     NETWORK_CHANNEL,
				     &ip, NETWORK_PORT,
				     &my_ip, NETWORK_LISTEN_PORT);
	iauth = iauth_create(network, IAUTH_MODULES);
}

static void proxy_deinit(void)
{
        if (lib_signal_kill != 0)
		i_warning("Killed with signal %d\n", lib_signal_kill);

	clients_disconnect("Proxy closed");

        iauth_deinit();

	client_commands_deinit();
	clients_deinit();
	servers_deinit();

	irc_network_destroy(network);
}

int main(int argc, char *argv[])
{
	tree_destroy(NULL); /* FIXME: annoying - libirc.a needs it, but ld tries to optimize it away.. */

        lib_init();
	ioloop = io_loop_create();

	proxy_init();
        io_loop_run(ioloop);
	proxy_deinit();

	io_loop_destroy(ioloop);
	lib_deinit();

        return 0;
}
